@@ -2,33 +2,24 @@
 """
 Python Frontmatter: Parse and manage posts with YAML frontmatter
 """
-from __future__ import unicode_literals
 
 import codecs
 import re
 
-import six
 
 from .util import u
 from .default_handlers import YAMLHandler, JSONHandler, TOMLHandler
 
 
-__all__ = ['parse', 'load', 'loads', 'dump', 'dumps']
+__all__ = ["parse", "load", "loads", "dump", "dumps"]
 
-POST_TEMPLATE = """\
-{start_delimiter}
-{metadata}
-{end_delimiter}
-
-{content}
-"""
 
 # global handlers
-handlers = {
-    Handler.FM_BOUNDARY: Handler() 
+handlers = [
+    Handler()
     for Handler in [YAMLHandler, JSONHandler, TOMLHandler]
     if Handler is not None
-}
+]
 
 
 def detect_format(text, handlers):
@@ -38,18 +29,18 @@ def detect_format(text, handlers):
 
     ``text`` should be unicode text about to be parsed.
 
-    ``handlers`` is a dictionary where keys are opening delimiters 
+    ``handlers`` is a dictionary where keys are opening delimiters
     and values are handler instances.
     """
-    for pattern, handler in handlers.items():
-        if pattern.match(text):
+    for handler in handlers:
+        if handler.detect(text):
             return handler
 
     # nothing matched, give nothing back
     return None
 
 
-def parse(text, encoding='utf-8', handler=None, **defaults):
+def parse(text, encoding="utf-8", handler=None, **defaults):
     """
     Parse text with frontmatter, return metadata and content.
     Pass in optional metadata defaults as keyword args.
@@ -57,9 +48,13 @@ def parse(text, encoding='utf-8', handler=None, **defaults):
     If frontmatter is not found, returns an empty metadata dictionary
     (or defaults) and original text content.
 
-    ::
+    .. testsetup:: *
 
-        >>> with open('tests/hello-world.markdown') as f:
+        >>> import frontmatter
+
+    .. doctest::
+
+        >>> with open('tests/yaml/hello-world.txt') as f:
         ...     metadata, content = frontmatter.parse(f.read())
         >>> print(metadata['title'])
         Hello, world!
@@ -91,36 +86,77 @@ def parse(text, encoding='utf-8', handler=None, **defaults):
     return metadata, content.strip()
 
 
-def load(fd, encoding='utf-8', handler=None, **defaults):
+def check(fd, encoding="utf-8"):
     """
-    Load and parse a file-like object or filename, 
-    return a :py:class:`post <frontmatter.Post>`.
+    Check if a file-like object or filename has a frontmatter,
+    return True if exists, False otherwise.
 
-    ::
+    If it contains a frontmatter but it is empty, return True as well.
 
-        >>> post = frontmatter.load('tests/hello-world.markdown')
-        >>> with open('tests/hello-world.markdown') as f:
-        ...     post = frontmatter.load(f)
+    .. doctest::
+
+        >>> frontmatter.check('tests/yaml/hello-world.txt')
+        True
 
     """
-    if hasattr(fd, 'read'):
+    if hasattr(fd, "read"):
         text = fd.read()
 
     else:
-        with codecs.open(fd, 'r', encoding) as f:
+        with codecs.open(fd, "r", encoding) as f:
+            text = f.read()
+
+    return checks(text, encoding)
+
+
+def checks(text, encoding="utf-8"):
+    """
+    Check if a text (binary or unicode) has a frontmatter,
+    return True if exists, False otherwise.
+
+    If it contains a frontmatter but it is empty, return True as well.
+
+    .. doctest::
+
+        >>> with open('tests/yaml/hello-world.txt') as f:
+        ...     frontmatter.checks(f.read())
+        True
+
+    """
+    text = u(text, encoding)
+    return detect_format(text, handlers) != None
+
+
+def load(fd, encoding="utf-8", handler=None, **defaults):
+    """
+    Load and parse a file-like object or filename,
+    return a :py:class:`post <frontmatter.Post>`.
+
+    .. doctest::
+
+        >>> post = frontmatter.load('tests/yaml/hello-world.txt')
+        >>> with open('tests/yaml/hello-world.txt') as f:
+        ...     post = frontmatter.load(f)
+
+    """
+    if hasattr(fd, "read"):
+        text = fd.read()
+
+    else:
+        with codecs.open(fd, "r", encoding) as f:
             text = f.read()
 
     handler = handler or detect_format(text, handlers)
     return loads(text, encoding, handler, **defaults)
 
 
-def loads(text, encoding='utf-8', handler=None, **defaults):
+def loads(text, encoding="utf-8", handler=None, **defaults):
     """
     Parse text (binary or unicode) and return a :py:class:`post <frontmatter.Post>`.
 
-    ::
+    .. doctest::
 
-        >>> with open('tests/hello-world.markdown') as f:
+        >>> with open('tests/yaml/hello-world.txt') as f:
         ...     post = frontmatter.loads(f.read())
 
     """
@@ -130,86 +166,117 @@ def loads(text, encoding='utf-8', handler=None, **defaults):
     return Post(content, handler, **metadata)
 
 
-def dump(post, fd, encoding='utf-8', handler=None, **kwargs):
+def dump(post, fd, encoding="utf-8", handler=None, **kwargs):
     """
     Serialize :py:class:`post <frontmatter.Post>` to a string and write to a file-like object.
     Text will be encoded on the way out (utf-8 by default).
 
     ::
 
-        >>> from io import StringIO
-        >>> f = StringIO()
+        >>> from io import BytesIO
+        >>> post = frontmatter.load('tests/yaml/hello-world.txt')
+        >>> f = BytesIO()
         >>> frontmatter.dump(post, f)
-        >>> print(f.getvalue())
+        >>> print(f.getvalue().decode('utf-8'))
         ---
-        excerpt: tl;dr
         layout: post
         title: Hello, world!
         ---
+        <BLANKLINE>
         Well, hello there, world.
 
 
+    .. testcode::
+
+        from io import BytesIO
+        post = frontmatter.load('tests/yaml/hello-world.txt')
+        f = BytesIO()
+        frontmatter.dump(post, f)
+        print(f.getvalue().decode('utf-8'))
+
+    .. testoutput::
+
+        ---
+        layout: post
+        title: Hello, world!
+        ---
+        <BLANKLINE>
+        Well, hello there, world.
+
     """
     content = dumps(post, handler, **kwargs)
-    if hasattr(fd, 'write'):
+    if hasattr(fd, "write"):
         fd.write(content.encode(encoding))
 
     else:
-        with codecs.open(fd, 'w', encoding) as f:
+        with codecs.open(fd, "w", encoding) as f:
             f.write(content)
 
 
 def dumps(post, handler=None, **kwargs):
     """
-    Serialize a :py:class:`post <frontmatter.Post>` to a string and return text. 
+    Serialize a :py:class:`post <frontmatter.Post>` to a string and return text.
     This always returns unicode text, which can then be encoded.
 
     Passing ``handler`` will change how metadata is turned into text. A handler
-    passed as an argument will override ``post.handler``, with 
-    :py:class:`YAMLHandler <frontmatter.default_handlers.YAMLHandler>` used as 
+    passed as an argument will override ``post.handler``, with
+    :py:class:`YAMLHandler <frontmatter.default_handlers.YAMLHandler>` used as
     a default.
+
     ::
 
-        >>> print(frontmatter.dumps(post))
+        >>> post = frontmatter.load('tests/yaml/hello-world.txt')
+        >>> print(frontmatter.dumps(post)) # doctest: +NORMALIZE_WHITESPACE
         ---
-        excerpt: tl;dr
         layout: post
         title: Hello, world!
         ---
+        <BLANKLINE>
+        Well, hello there, world.
+
+    .. testcode::
+
+        post = frontmatter.load('tests/yaml/hello-world.txt')
+        print(frontmatter.dumps(post))
+
+    .. testoutput::
+
+        ---
+        layout: post
+        title: Hello, world!
+        ---
+
         Well, hello there, world.
 
     """
     if handler is None:
-        handler = getattr(post, 'handler', None) or YAMLHandler()
+        handler = getattr(post, "handler", None) or YAMLHandler()
 
-    start_delimiter = kwargs.pop('start_delimiter', handler.START_DELIMITER)
-    end_delimiter = kwargs.pop('end_delimiter', handler.END_DELIMITER)
-
-    metadata = handler.export(post.metadata, **kwargs)
-
-    return POST_TEMPLATE.format(
-        metadata=metadata, content=post.content,
-        start_delimiter=start_delimiter,
-        end_delimiter=end_delimiter).strip()
+    return handler.format(post, **kwargs)
 
 
 class Post(object):
     """
     A post contains content and metadata from Front Matter. This is what gets
-    returned by :py:func:`load <frontmatter.load>` and :py:func:`loads <frontmatter.loads>`. 
-    Passing this to :py:func:`dump <frontmatter.dump>` or :py:func:`dumps <frontmatter.dumps>` 
+    returned by :py:func:`load <frontmatter.load>` and :py:func:`loads <frontmatter.loads>`.
+    Passing this to :py:func:`dump <frontmatter.dump>` or :py:func:`dumps <frontmatter.dumps>`
     will turn it back into text.
 
-    For convenience, metadata values are available as proxied item lookups. 
+    For convenience, metadata values are available as proxied item lookups.
     """
+
     def __init__(self, content, handler=None, **metadata):
-        self.content = u(content)
+        self.content = str(content)
         self.metadata = metadata
         self.handler = handler
 
     def __getitem__(self, name):
         "Get metadata key"
-        return self.metadata[name]        
+        return self.metadata[name]
+
+    def __contains__(self, item):
+        "Check metadata contains key"
+        return item in self.metadata
 
     def __setitem__(self, name, value):
         "Set a metadata key"
@@ -220,14 +287,9 @@ class Post(object):
         del self.metadata[name]
 
     def __bytes__(self):
-        return self.content.encode('utf-8')
+        return self.content.encode("utf-8")
 
     def __str__(self):
-        if six.PY2:
-            return self.__bytes__()
-        return self.content
-
-    def __unicode__(self):
         return self.content
 
     def get(self, key, default=None):
@@ -245,6 +307,5 @@ class Post(object):
     def to_dict(self):
         "Post as a dict, for serializing"
         d = self.metadata.copy()
-        d['content'] = self.content
+        d["content"] = self.content
         return d
-
