@@ -2,14 +2,14 @@
 """
 Utilities for handling unicode and other repetitive bits
 """
-import six
 
-def u(text, encoding='utf-8'):
+
+def u(text, encoding="utf-8"):
     "Return unicode text, no matter what"
 
-    if isinstance(text, six.binary_type):
+    if isinstance(text, bytes):
         text = text.decode(encoding)
 
     # it's already unicode
-    text = text.replace('\r\n', '\n')
-    return text
\ No newline at end of file
+    text = text.replace("\r\n", "\n")
+    return text
